(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 13.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     73143,       1203]
NotebookOptionsPosition[     70709,       1158]
NotebookOutlinePosition[     71131,       1175]
CellTagsIndexPosition[     71088,       1172]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
 StyleBox["appcg",
  FontColor->RGBColor[0.5, 0, 0.5]],
 " job notebook"
}], "Title",
 CellChangeTimes->{{3.856165344530184*^9, 3.856165360586437*^9}, {
   3.856592043342741*^9, 3.8565920465891027`*^9}, {3.860951635120672*^9, 
   3.860951644840625*^9}, 
   3.860958941586567*^9},ExpressionUUID->"77b3805d-060b-4070-8508-\
41576cdf0004"],

Cell["\<\
This notebook generates a Wolfram language file to be run on \
appcg.ra.phy.cam.ac.uk, a dedicated compute server belonging to the Cavendish \
Laboratory Astrophysics Group.\
\>", "Text",
 CellChangeTimes->{{3.85659205646922*^9, 3.8565920652907*^9}, {
  3.86095894999116*^9, 
  3.8609590094947367`*^9}},ExpressionUUID->"ac3b952a-323b-405e-9ac1-\
f072cdc53b32"],

Cell["\<\
We use the $Timing environment to activate some (undocumented) functionality \
in HiGGS, which clocks function calls and allows us to plot the HPC \
electroencephalograph (EEG). Beware, this is a very inefficient and data - \
intensive process!\
\>", "Text",
 CellChangeTimes->{{3.861995369715053*^9, 
  3.861995373031034*^9}},ExpressionUUID->"c9fedf3e-88c2-4938-860c-\
5810784e7e18"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Timing", "=", "True"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.860694729913267*^9, 3.8606947345521383`*^9}, {
  3.860941328673038*^9, 
  3.860941330424848*^9}},ExpressionUUID->"9966b0f6-fed7-4946-93a6-\
692cd3f2588c"],

Cell["\<\
We invent a \[OpenCurlyDoubleQuote]node\[CloseCurlyDoubleQuote] on which the \
job is run: in reality appcg is a single compute node.\
\>", "Text",
 CellChangeTimes->{{3.860964405708005*^9, 
  3.8609644287361097`*^9}},ExpressionUUID->"d26fa4bd-c3f3-4a56-9cd8-\
37b9ae8b758b"],

Cell[BoxData[
 RowBox[{"$Node", "=", "\"\<0\>\""}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.860964439088923*^9, 3.8609644508241177`*^9}, {
  3.861016228743435*^9, 
  3.861016228959337*^9}},ExpressionUUID->"fd726b7b-eb57-4386-8546-\
8b40abb5b147"],

Cell["Purge the survey directory.", "Text",
 CellChangeTimes->{{3.860964492151361*^9, 
  3.860964498663333*^9}},ExpressionUUID->"28427427-46cf-4df7-8bfc-\
e4b50069fca9"],

Cell[BoxData[{
 RowBox[{"Run", "@", "\"\<rm -rf ./svy/node-*\>\""}], "\[IndentingNewLine]", 
 RowBox[{"Run", "@", "\"\<rm -rf ./svy\>\""}], "\[IndentingNewLine]", 
 RowBox[{"Run", "@", "\"\<mkdir ./svy\>\""}], "\[IndentingNewLine]", 
 RowBox[{"Run", "@", "\"\<mkdir ./svy/node-0\>\""}], "\[IndentingNewLine]", 
 RowBox[{"Run", "@", "\"\<mkdir ./svy/node-0/chr\>\""}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.860399987586609*^9, 3.860400012970598*^9}, {
  3.860429185749961*^9, 3.860429188165661*^9}, {3.860429233165124*^9, 
  3.860429246557065*^9}, {3.860963072662879*^9, 3.8609630732779293`*^9}, {
  3.860963191132722*^9, 3.860963202620644*^9}, {3.8609645027436543`*^9, 
  3.8609645119274178`*^9}, {3.86101499051589*^9, 3.861014992475698*^9}, {
  3.8610164105500107`*^9, 
  3.861016412829463*^9}},ExpressionUUID->"ee8cde45-7297-47ec-8e7b-\
ec674b3b00cf"],

Cell["Load and build HiGGS.", "Text",
 CellChangeTimes->{{3.862038548499639*^9, 
  3.862038551490259*^9}},ExpressionUUID->"38aec83e-3336-41f1-ac4d-\
fd2593bf2789"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"Load", " ", "HiGGS"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Needs", "[", "\"\<xAct`HiGGS`\>\"", "]"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", 
    RowBox[{"Build", " ", "the", " ", "HiGGS", " ", "environment"}], "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"BuildHiGGS", "[", "]"}], ";"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.8561134940016623`*^9, 3.856113563953445*^9}, {
   3.856113864152964*^9, 3.856113866735909*^9}, {3.856114686146227*^9, 
   3.856114722201364*^9}, {3.8561147534818153`*^9, 3.856114786785334*^9}, {
   3.8561150379530697`*^9, 3.8561150382066393`*^9}, {3.856115132575592*^9, 
   3.856115149455024*^9}, 3.856165332453063*^9, {3.856165412953887*^9, 
   3.856165493969346*^9}, {3.856165882551499*^9, 3.8561659681267014`*^9}, {
   3.856252550417321*^9, 3.856252550671973*^9}, {3.856253384785056*^9, 
   3.856253537238635*^9}, {3.856253871827731*^9, 3.856253912977112*^9}, {
   3.856266135351696*^9, 3.85626613949498*^9}, {3.856330112716134*^9, 
   3.856330148356846*^9}, {3.856334077806583*^9, 3.85633412540273*^9}, {
   3.8563374211690483`*^9, 3.8563375146333323`*^9}, {3.8563713031675777`*^9, 
   3.8563713066542683`*^9}, {3.856407380427725*^9, 3.856407391803412*^9}, {
   3.856407608764429*^9, 3.8564076196720247`*^9}, 3.856407752091467*^9, 
   3.856407826016653*^9, 3.856407902750139*^9, {3.85640794860667*^9, 
   3.8564079506166077`*^9}, {3.8564080331074467`*^9, 3.856408035278562*^9}, 
   3.856408749583098*^9, {3.8564088204491577`*^9, 3.856408851243552*^9}, {
   3.856408937610999*^9, 3.856408958571206*^9}, 3.856426594371249*^9, {
   3.856546771934794*^9, 3.856546780533009*^9}, {3.856547050566327*^9, 
   3.856547182411356*^9}, {3.8565472180334578`*^9, 3.8565472297545033`*^9}, {
   3.856547261019403*^9, 3.856547319535778*^9}, {3.856547389634677*^9, 
   3.856547415811049*^9}, {3.856547502452675*^9, 3.8565475402935333`*^9}, {
   3.856547935615954*^9, 3.856547996958641*^9}, {3.8565480427427683`*^9, 
   3.856548130126974*^9}, {3.856548174271737*^9, 3.856548238495077*^9}, {
   3.856548994305244*^9, 3.856549018994968*^9}, {3.856549639533609*^9, 
   3.856549677197238*^9}, {3.856550750202263*^9, 3.856550754237236*^9}, {
   3.856584484801955*^9, 3.856584543913227*^9}, {3.856584817941999*^9, 
   3.856584869979888*^9}, 3.856585073668594*^9, {3.85658620385732*^9, 
   3.856586205956044*^9}, {3.856588920521818*^9, 3.856588921557769*^9}, 
   3.856588960476514*^9, {3.856590039993527*^9, 3.85659006622727*^9}, {
   3.856593936282866*^9, 3.856593948507574*^9}, {3.856593995428607*^9, 
   3.856593999792821*^9}, {3.856594042031241*^9, 3.856594053246723*^9}, {
   3.856594086439999*^9, 3.856594226084935*^9}, {3.85659432821384*^9, 
   3.856594469940914*^9}, {3.856594500282078*^9, 3.856594544618071*^9}, {
   3.856594612259853*^9, 3.856594769070032*^9}, {3.856594873905396*^9, 
   3.856594956693438*^9}, {3.8565950194680347`*^9, 3.856595074756282*^9}, {
   3.856595284751771*^9, 3.856595285285953*^9}, {3.856595383481922*^9, 
   3.856595426680007*^9}, {3.856596034870791*^9, 3.8565960350298347`*^9}, {
   3.85659607209651*^9, 3.85659608586699*^9}, {3.8565965701300287`*^9, 
   3.856596638947892*^9}, {3.856596725375925*^9, 3.856596731990838*^9}, {
   3.856598251369217*^9, 3.856598258241246*^9}, {3.856598434570442*^9, 
   3.856598462529182*^9}, {3.85659873835382*^9, 3.85659873866574*^9}, 
   3.856598824236*^9, {3.856598867332992*^9, 3.856598962700162*^9}, {
   3.8565990452773323`*^9, 3.856599068467422*^9}, {3.8565991551085377`*^9, 
   3.8565993229007053`*^9}, {3.8565993536914263`*^9, 3.856599417248561*^9}, {
   3.856600138751534*^9, 3.8566001661422253`*^9}, {3.8566002060986834`*^9, 
   3.856600225334676*^9}, {3.856600544026388*^9, 3.8566005441484118`*^9}, {
   3.856600586409638*^9, 3.856600612088909*^9}, {3.856600663947249*^9, 
   3.856600689908874*^9}, {3.856600745930562*^9, 3.8566008026048613`*^9}, {
   3.8566010638589973`*^9, 3.856601092986932*^9}, {3.85660138015917*^9, 
   3.8566013877983093`*^9}, {3.856606601818824*^9, 3.856606632750181*^9}, {
   3.856606721016076*^9, 3.856606801080719*^9}, {3.856607075354292*^9, 
   3.8566070793089247`*^9}, {3.856607285519328*^9, 3.856607350667756*^9}, {
   3.8566074599074087`*^9, 3.85660763592397*^9}, {3.856615179405943*^9, 
   3.856615210040922*^9}, {3.856630195894566*^9, 3.856630253259329*^9}, {
   3.8566931344471893`*^9, 3.856693204813795*^9}, 3.856693305118594*^9, {
   3.856693563355632*^9, 3.856693567190674*^9}, {3.856693669447075*^9, 
   3.8566936779144287`*^9}, {3.8566938062708397`*^9, 3.856693808318098*^9}, {
   3.8566939398466473`*^9, 3.856693942230942*^9}, {3.856697869620206*^9, 
   3.856697873773686*^9}, {3.856698360189404*^9, 3.856698371872653*^9}, {
   3.856700875553402*^9, 3.8567008771913757`*^9}, {3.8567019896412363`*^9, 
   3.856701992163899*^9}, {3.856702058555892*^9, 3.856702093022097*^9}, {
   3.8567024693164377`*^9, 3.856702472672245*^9}, {3.856702662221097*^9, 
   3.8567026639724407`*^9}, {3.856702933353261*^9, 3.856702937571703*^9}, {
   3.856702969037169*^9, 3.8567029816901608`*^9}, {3.856703272825307*^9, 
   3.8567032770478907`*^9}, {3.856703434516992*^9, 3.856703449079845*^9}, {
   3.856703764231814*^9, 3.856703769679906*^9}, {3.85670523350788*^9, 
   3.856705255121399*^9}, {3.856705300802005*^9, 3.8567053440432377`*^9}, 
   3.8567053835308113`*^9, {3.856716776053294*^9, 3.856716798617839*^9}, {
   3.856717296675805*^9, 3.856717300454899*^9}, {3.856717455235383*^9, 
   3.856717459302205*^9}, {3.856717558228253*^9, 3.856717577490952*^9}, {
   3.856766079861846*^9, 3.856766199328062*^9}, {3.856768657046853*^9, 
   3.856768658811205*^9}, {3.856768689609551*^9, 3.8567687186879473`*^9}, {
   3.856847824059163*^9, 3.856847829756719*^9}, {3.856885056781955*^9, 
   3.8568850572425117`*^9}, {3.8568875962693453`*^9, 3.856887605235325*^9}, {
   3.8569364364172297`*^9, 3.856936438322406*^9}, {3.8569572256026897`*^9, 
   3.856957232048373*^9}, {3.858142966335279*^9, 3.858142967860476*^9}, {
   3.85818112692178*^9, 3.8581811377360287`*^9}, {3.858181271677498*^9, 
   3.8581812782539043`*^9}, 3.858181354071229*^9, {3.858181524199338*^9, 
   3.858181555061183*^9}, 3.858181602086733*^9, {3.858181671389921*^9, 
   3.858181758453566*^9}, {3.858181924587641*^9, 3.858181926451866*^9}, {
   3.858181962742115*^9, 3.858181972902639*^9}, {3.8581820085582523`*^9, 
   3.8581820428319693`*^9}, {3.858182313602854*^9, 3.858182318711783*^9}, 
   3.858182736132729*^9, {3.858182873156907*^9, 3.858182876713172*^9}, {
   3.858184058358448*^9, 3.858184060194497*^9}, {3.858184214826552*^9, 
   3.8581842270070047`*^9}, {3.858184821147771*^9, 3.858184822231308*^9}, {
   3.8581858194322453`*^9, 3.85818582401626*^9}, {3.8582267013467503`*^9, 
   3.8582267055196037`*^9}, 3.858228301091529*^9, {3.858229492565473*^9, 
   3.8582295062000647`*^9}, {3.8582296669434967`*^9, 
   3.8582297318074417`*^9}, {3.8582301400178213`*^9, 
   3.8582301519399233`*^9}, {3.8582307301490602`*^9, 3.858230772531424*^9}, {
   3.858231510513658*^9, 3.858231523827112*^9}, {3.858233464638513*^9, 
   3.8582334712864532`*^9}, {3.8582336759973717`*^9, 
   3.8582336788817043`*^9}, {3.8582343296727877`*^9, 3.858234401096449*^9}, 
   3.858234510434876*^9, {3.858235066237193*^9, 3.8582350738367023`*^9}, {
   3.858235202140679*^9, 3.858235220235373*^9}, {3.858235454699109*^9, 
   3.858235467892789*^9}, {3.858236200960024*^9, 3.858236217929487*^9}, {
   3.858236832559057*^9, 3.858236840867004*^9}, {3.858238575009941*^9, 
   3.85823861822012*^9}, {3.8582393686107492`*^9, 3.8582393946451683`*^9}, {
   3.858239531711216*^9, 3.858239538715068*^9}, {3.8582397821401167`*^9, 
   3.858239782385582*^9}, {3.858243501836114*^9, 3.858243533066533*^9}, 
   3.8582517677726316`*^9, {3.858253141465578*^9, 3.858253156004655*^9}, {
   3.858262523424046*^9, 3.8582625263262672`*^9}, {3.8582626132154093`*^9, 
   3.858262651990097*^9}, {3.858263093996423*^9, 3.858263107363203*^9}, {
   3.858269621613408*^9, 3.8582696456949043`*^9}, {3.858270163378757*^9, 
   3.858270173825705*^9}, {3.8582710242137823`*^9, 3.858271030036566*^9}, {
   3.858339575377823*^9, 3.858339606721662*^9}, {3.858342927878196*^9, 
   3.8583429658853207`*^9}, {3.858433836920229*^9, 3.8584338580792007`*^9}, {
   3.858444408381193*^9, 3.858444410492931*^9}, {3.858488493308608*^9, 
   3.858488497917211*^9}, {3.858488536588729*^9, 3.8584885507801733`*^9}, {
   3.858489530903399*^9, 3.858489533822534*^9}, {3.858490078702059*^9, 
   3.858490081194944*^9}, {3.8584935454650297`*^9, 3.858493546463683*^9}, {
   3.858493586904846*^9, 3.8584936478150787`*^9}, {3.858495601141667*^9, 
   3.858495607299471*^9}, {3.858496325240609*^9, 3.858496327438936*^9}, 
   3.858497286249983*^9, {3.858498767025363*^9, 3.858498795672372*^9}, {
   3.858499119271175*^9, 3.8584991240866756`*^9}, {3.858510475487196*^9, 
   3.8585104773659773`*^9}, {3.858510663777513*^9, 3.8585107203833313`*^9}, {
   3.858510766166922*^9, 3.858510819231451*^9}, {3.858514732542738*^9, 
   3.858514775339328*^9}, {3.858524787588656*^9, 3.858524805815624*^9}, {
   3.858586280769*^9, 3.858586320902526*^9}, {3.8586041766885843`*^9, 
   3.858604235656663*^9}, {3.858604268024601*^9, 3.8586043932366858`*^9}, {
   3.8586756463415127`*^9, 3.858675658869523*^9}, {3.858693513549828*^9, 
   3.858693520322974*^9}, {3.858707622178495*^9, 3.858707625882985*^9}, {
   3.858713068848999*^9, 3.858713111616528*^9}, {3.8587397670160303`*^9, 
   3.85873976882025*^9}, {3.8587675401932583`*^9, 3.858767541652359*^9}, {
   3.858832057428598*^9, 3.858832057926138*^9}, {3.858837232206874*^9, 
   3.858837233645618*^9}, {3.858853493013969*^9, 3.8588535088894873`*^9}, 
   3.858855124164269*^9, {3.8588564226327744`*^9, 3.858856431548901*^9}, 
   3.8589136332742987`*^9, 3.858919114696944*^9, 3.85893209743512*^9, 
   3.858945274840987*^9, {3.858952938922271*^9, 3.858952942944778*^9}, 
   3.8589531113849363`*^9, 3.858995404369176*^9, {3.8589956417225437`*^9, 
   3.858995644914462*^9}, {3.8590358762954283`*^9, 3.85903590700359*^9}, {
   3.859091432673786*^9, 3.8590914661578608`*^9}, 3.859117315940999*^9, 
   3.859146655027433*^9, {3.859172984765532*^9, 3.859172993171767*^9}, {
   3.8592159531931887`*^9, 3.859215953986575*^9}, {3.859617013949033*^9, 
   3.8596170867982264`*^9}, {3.85970281276968*^9, 3.85970281470197*^9}, {
   3.859742016589634*^9, 3.8597420216673613`*^9}, {3.8597835044637527`*^9, 
   3.8597835199839087`*^9}, {3.8597911002859983`*^9, 3.859791100724319*^9}, 
   3.860040870391541*^9, {3.860944114979669*^9, 3.860944123421249*^9}, {
   3.860944933263855*^9, 3.860944945175519*^9}, {3.8609449945269814`*^9, 
   3.860945023815143*^9}, {3.860946122451845*^9, 3.8609461255880117`*^9}, {
   3.860951081407662*^9, 3.860951112256353*^9}, {3.860952240732245*^9, 
   3.860952251988269*^9}, {3.860952330947336*^9, 3.860952331211688*^9}, {
   3.860958306928714*^9, 3.860958310488752*^9}, {3.860964166172598*^9, 
   3.860964166892364*^9}, {3.860964225686287*^9, 
   3.860964226675321*^9}},ExpressionUUID->"ecfae8f8-9928-4519-ae15-\
3f06f02b8c48"],

Cell["Einstein-Cartan theory.", "Text",
 CellChangeTimes->{{3.8609642987533092`*^9, 
  3.860964306281268*^9}},ExpressionUUID->"a8923ab6-e09d-46fe-bae0-\
5887d4929536"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"JobsBatch", "=", 
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"\"\<EinsteinCartan\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Alp1", "==", "0"}], ",", 
          RowBox[{"Alp2", "==", "0"}], ",", 
          RowBox[{"Alp3", "==", "0"}], ",", 
          RowBox[{"Alp4", "==", "0"}], ",", 
          RowBox[{"Alp5", "==", "0"}], ",", 
          RowBox[{"Alp6", "==", "0"}], ",", 
          RowBox[{"Bet1", "==", "0"}], ",", 
          RowBox[{"Bet2", "==", "0"}], ",", 
          RowBox[{"Bet3", "==", "0"}], ",", 
          RowBox[{"cAlp1", "==", "0"}], ",", 
          RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], "}"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", "*)"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.8561134940016623`*^9, 3.856113563953445*^9}, {
   3.856113864152964*^9, 3.856113866735909*^9}, {3.856114686146227*^9, 
   3.856114722201364*^9}, {3.8561147534818153`*^9, 3.856114786785334*^9}, {
   3.8561150379530697`*^9, 3.8561150382066393`*^9}, {3.856115132575592*^9, 
   3.856115149455024*^9}, 3.856165332453063*^9, {3.856165412953887*^9, 
   3.856165493969346*^9}, {3.856165882551499*^9, 3.8561659681267014`*^9}, {
   3.856252550417321*^9, 3.856252550671973*^9}, {3.856253384785056*^9, 
   3.856253537238635*^9}, {3.856253871827731*^9, 3.856253912977112*^9}, {
   3.856266135351696*^9, 3.85626613949498*^9}, {3.856330112716134*^9, 
   3.856330148356846*^9}, {3.856334077806583*^9, 3.85633412540273*^9}, {
   3.8563374211690483`*^9, 3.8563375146333323`*^9}, {3.8563713031675777`*^9, 
   3.8563713066542683`*^9}, {3.856407380427725*^9, 3.856407391803412*^9}, {
   3.856407608764429*^9, 3.8564076196720247`*^9}, 3.856407752091467*^9, 
   3.856407826016653*^9, 3.856407902750139*^9, {3.85640794860667*^9, 
   3.8564079506166077`*^9}, {3.8564080331074467`*^9, 3.856408035278562*^9}, 
   3.856408749583098*^9, {3.8564088204491577`*^9, 3.856408851243552*^9}, {
   3.856408937610999*^9, 3.856408958571206*^9}, 3.856426594371249*^9, {
   3.856546771934794*^9, 3.856546780533009*^9}, {3.856547050566327*^9, 
   3.856547182411356*^9}, {3.8565472180334578`*^9, 3.8565472297545033`*^9}, {
   3.856547261019403*^9, 3.856547319535778*^9}, {3.856547389634677*^9, 
   3.856547415811049*^9}, {3.856547502452675*^9, 3.8565475402935333`*^9}, {
   3.856547935615954*^9, 3.856547996958641*^9}, {3.8565480427427683`*^9, 
   3.856548130126974*^9}, {3.856548174271737*^9, 3.856548238495077*^9}, {
   3.856548994305244*^9, 3.856549018994968*^9}, {3.856549639533609*^9, 
   3.856549677197238*^9}, {3.856550750202263*^9, 3.856550754237236*^9}, {
   3.856584484801955*^9, 3.856584543913227*^9}, {3.856584817941999*^9, 
   3.856584869979888*^9}, 3.856585073668594*^9, {3.85658620385732*^9, 
   3.856586205956044*^9}, {3.856588920521818*^9, 3.856588921557769*^9}, 
   3.856588960476514*^9, {3.856590039993527*^9, 3.85659006622727*^9}, {
   3.856593936282866*^9, 3.856593948507574*^9}, {3.856593995428607*^9, 
   3.856593999792821*^9}, {3.856594042031241*^9, 3.856594053246723*^9}, {
   3.856594086439999*^9, 3.856594226084935*^9}, {3.85659432821384*^9, 
   3.856594469940914*^9}, {3.856594500282078*^9, 3.856594544618071*^9}, {
   3.856594612259853*^9, 3.856594769070032*^9}, {3.856594873905396*^9, 
   3.856594956693438*^9}, {3.8565950194680347`*^9, 3.856595074756282*^9}, {
   3.856595284751771*^9, 3.856595285285953*^9}, {3.856595383481922*^9, 
   3.856595426680007*^9}, {3.856596034870791*^9, 3.8565960350298347`*^9}, {
   3.85659607209651*^9, 3.85659608586699*^9}, {3.8565965701300287`*^9, 
   3.856596638947892*^9}, {3.856596725375925*^9, 3.856596731990838*^9}, {
   3.856598251369217*^9, 3.856598258241246*^9}, {3.856598434570442*^9, 
   3.856598462529182*^9}, {3.85659873835382*^9, 3.85659873866574*^9}, 
   3.856598824236*^9, {3.856598867332992*^9, 3.856598962700162*^9}, {
   3.8565990452773323`*^9, 3.856599068467422*^9}, {3.8565991551085377`*^9, 
   3.8565993229007053`*^9}, {3.8565993536914263`*^9, 3.856599417248561*^9}, {
   3.856600138751534*^9, 3.8566001661422253`*^9}, {3.8566002060986834`*^9, 
   3.856600225334676*^9}, {3.856600544026388*^9, 3.8566005441484118`*^9}, {
   3.856600586409638*^9, 3.856600612088909*^9}, {3.856600663947249*^9, 
   3.856600689908874*^9}, {3.856600745930562*^9, 3.8566008026048613`*^9}, {
   3.8566010638589973`*^9, 3.856601092986932*^9}, {3.85660138015917*^9, 
   3.8566013877983093`*^9}, {3.856606601818824*^9, 3.856606632750181*^9}, {
   3.856606721016076*^9, 3.856606801080719*^9}, {3.856607075354292*^9, 
   3.8566070793089247`*^9}, {3.856607285519328*^9, 3.856607350667756*^9}, {
   3.8566074599074087`*^9, 3.85660763592397*^9}, {3.856615179405943*^9, 
   3.856615210040922*^9}, {3.856630195894566*^9, 3.856630253259329*^9}, {
   3.8566931344471893`*^9, 3.856693204813795*^9}, 3.856693305118594*^9, {
   3.856693563355632*^9, 3.856693567190674*^9}, {3.856693669447075*^9, 
   3.8566936779144287`*^9}, {3.8566938062708397`*^9, 3.856693808318098*^9}, {
   3.8566939398466473`*^9, 3.856693942230942*^9}, {3.856697869620206*^9, 
   3.856697873773686*^9}, {3.856698360189404*^9, 3.856698371872653*^9}, {
   3.856700875553402*^9, 3.8567008771913757`*^9}, {3.8567019896412363`*^9, 
   3.856701992163899*^9}, {3.856702058555892*^9, 3.856702093022097*^9}, {
   3.8567024693164377`*^9, 3.856702472672245*^9}, {3.856702662221097*^9, 
   3.8567026639724407`*^9}, {3.856702933353261*^9, 3.856702937571703*^9}, {
   3.856702969037169*^9, 3.8567029816901608`*^9}, {3.856703272825307*^9, 
   3.8567032770478907`*^9}, {3.856703434516992*^9, 3.856703449079845*^9}, {
   3.856703764231814*^9, 3.856703769679906*^9}, {3.85670523350788*^9, 
   3.856705255121399*^9}, {3.856705300802005*^9, 3.8567053440432377`*^9}, 
   3.8567053835308113`*^9, {3.856716776053294*^9, 3.856716798617839*^9}, {
   3.856717296675805*^9, 3.856717300454899*^9}, {3.856717455235383*^9, 
   3.856717459302205*^9}, {3.856717558228253*^9, 3.856717577490952*^9}, {
   3.856766079861846*^9, 3.856766199328062*^9}, {3.856768657046853*^9, 
   3.856768658811205*^9}, {3.856768689609551*^9, 3.8567687186879473`*^9}, {
   3.856847824059163*^9, 3.856847829756719*^9}, {3.856885056781955*^9, 
   3.8568850572425117`*^9}, {3.8568875962693453`*^9, 3.856887605235325*^9}, {
   3.8569364364172297`*^9, 3.856936438322406*^9}, {3.8569572256026897`*^9, 
   3.856957232048373*^9}, {3.858142966335279*^9, 3.858142967860476*^9}, {
   3.85818112692178*^9, 3.8581811377360287`*^9}, {3.858181271677498*^9, 
   3.8581812782539043`*^9}, 3.858181354071229*^9, {3.858181524199338*^9, 
   3.858181555061183*^9}, 3.858181602086733*^9, {3.858181671389921*^9, 
   3.858181758453566*^9}, {3.858181924587641*^9, 3.858181926451866*^9}, {
   3.858181962742115*^9, 3.858181972902639*^9}, {3.8581820085582523`*^9, 
   3.8581820428319693`*^9}, {3.858182313602854*^9, 3.858182318711783*^9}, 
   3.858182736132729*^9, {3.858182873156907*^9, 3.858182876713172*^9}, {
   3.858184058358448*^9, 3.858184060194497*^9}, {3.858184214826552*^9, 
   3.8581842270070047`*^9}, {3.858184821147771*^9, 3.858184822231308*^9}, {
   3.8581858194322453`*^9, 3.85818582401626*^9}, {3.8582267013467503`*^9, 
   3.8582267055196037`*^9}, 3.858228301091529*^9, {3.858229492565473*^9, 
   3.8582295062000647`*^9}, {3.8582296669434967`*^9, 
   3.8582297318074417`*^9}, {3.8582301400178213`*^9, 
   3.8582301519399233`*^9}, {3.8582307301490602`*^9, 3.858230772531424*^9}, {
   3.858231510513658*^9, 3.858231523827112*^9}, {3.858233464638513*^9, 
   3.8582334712864532`*^9}, {3.8582336759973717`*^9, 
   3.8582336788817043`*^9}, {3.8582343296727877`*^9, 3.858234401096449*^9}, 
   3.858234510434876*^9, {3.858235066237193*^9, 3.8582350738367023`*^9}, {
   3.858235202140679*^9, 3.858235220235373*^9}, {3.858235454699109*^9, 
   3.858235467892789*^9}, {3.858236200960024*^9, 3.858236217929487*^9}, {
   3.858236832559057*^9, 3.858236840867004*^9}, {3.858238575009941*^9, 
   3.85823861822012*^9}, {3.8582393686107492`*^9, 3.8582393946451683`*^9}, {
   3.858239531711216*^9, 3.858239538715068*^9}, {3.8582397821401167`*^9, 
   3.858239782385582*^9}, {3.858243501836114*^9, 3.858243533066533*^9}, 
   3.8582517677726316`*^9, {3.858253141465578*^9, 3.858253156004655*^9}, {
   3.858262523424046*^9, 3.8582625263262672`*^9}, {3.8582626132154093`*^9, 
   3.858262651990097*^9}, {3.858263093996423*^9, 3.858263107363203*^9}, {
   3.858269621613408*^9, 3.8582696456949043`*^9}, {3.858270163378757*^9, 
   3.858270173825705*^9}, {3.8582710242137823`*^9, 3.858271030036566*^9}, {
   3.858339575377823*^9, 3.858339606721662*^9}, {3.858342927878196*^9, 
   3.8583429658853207`*^9}, {3.858433836920229*^9, 3.8584338580792007`*^9}, {
   3.858444408381193*^9, 3.858444410492931*^9}, {3.858488493308608*^9, 
   3.858488497917211*^9}, {3.858488536588729*^9, 3.8584885507801733`*^9}, {
   3.858489530903399*^9, 3.858489533822534*^9}, {3.858490078702059*^9, 
   3.858490081194944*^9}, {3.8584935454650297`*^9, 3.858493546463683*^9}, {
   3.858493586904846*^9, 3.8584936478150787`*^9}, {3.858495601141667*^9, 
   3.858495607299471*^9}, {3.858496325240609*^9, 3.858496327438936*^9}, 
   3.858497286249983*^9, {3.858498767025363*^9, 3.858498795672372*^9}, {
   3.858499119271175*^9, 3.8584991240866756`*^9}, {3.858510475487196*^9, 
   3.8585104773659773`*^9}, {3.858510663777513*^9, 3.8585107203833313`*^9}, {
   3.858510766166922*^9, 3.858510819231451*^9}, {3.858514732542738*^9, 
   3.858514775339328*^9}, {3.858524787588656*^9, 3.858524805815624*^9}, {
   3.858586280769*^9, 3.858586320902526*^9}, {3.8586041766885843`*^9, 
   3.858604235656663*^9}, {3.858604268024601*^9, 3.8586043932366858`*^9}, {
   3.8586756463415127`*^9, 3.858675658869523*^9}, {3.858693513549828*^9, 
   3.858693520322974*^9}, {3.858707622178495*^9, 3.858707625882985*^9}, {
   3.858713068848999*^9, 3.858713111616528*^9}, {3.8587397670160303`*^9, 
   3.85873976882025*^9}, {3.8587675401932583`*^9, 3.858767541652359*^9}, {
   3.858832057428598*^9, 3.858832057926138*^9}, {3.858837232206874*^9, 
   3.858837233645618*^9}, {3.858853493013969*^9, 3.8588535088894873`*^9}, 
   3.858855124164269*^9, {3.8588564226327744`*^9, 3.858856431548901*^9}, 
   3.8589136332742987`*^9, 3.858919114696944*^9, 3.85893209743512*^9, 
   3.858945274840987*^9, {3.858952938922271*^9, 3.858952942944778*^9}, 
   3.8589531113849363`*^9, 3.858995404369176*^9, {3.8589956417225437`*^9, 
   3.858995644914462*^9}, {3.8590358762954283`*^9, 3.85903590700359*^9}, {
   3.859091432673786*^9, 3.8590914661578608`*^9}, 3.859117315940999*^9, 
   3.859146655027433*^9, {3.859172984765532*^9, 3.859172993171767*^9}, {
   3.8592159531931887`*^9, 3.859215953986575*^9}, {3.859617013949033*^9, 
   3.8596170867982264`*^9}, {3.85970281276968*^9, 3.85970281470197*^9}, {
   3.859742016589634*^9, 3.8597420216673613`*^9}, {3.8597835044637527`*^9, 
   3.8597835199839087`*^9}, {3.8597911002859983`*^9, 3.859791100724319*^9}, 
   3.860040870391541*^9, {3.860944114979669*^9, 3.860944123421249*^9}, {
   3.860944933263855*^9, 3.860944945175519*^9}, {3.8609449945269814`*^9, 
   3.860945023815143*^9}, {3.860946122451845*^9, 3.8609461255880117`*^9}, {
   3.860951081407662*^9, 3.860951112256353*^9}, {3.860952240732245*^9, 
   3.860952251988269*^9}, {3.860952330947336*^9, 3.860952331211688*^9}, {
   3.860958306928714*^9, 3.860958310488752*^9}, {3.860964166172598*^9, 
   3.860964166892364*^9}, {3.860964225686287*^9, 3.860964226675321*^9}, {
   3.86096426922003*^9, 3.86096427095472*^9}, {3.861077044060281*^9, 
   3.8610770548677607`*^9}, {3.861119273806775*^9, 3.861119275286681*^9}, {
   3.862038588693138*^9, 3.8620385888264523`*^9}, {3.862038761587306*^9, 
   3.862038761786759*^9}, {3.862306272763246*^9, 3.862306274762842*^9}, {
   3.86230708930536*^9, 
   3.862307090937265*^9}},ExpressionUUID->"ffea50e1-9566-4d73-8e16-\
9677a199ce2a"],

Cell["Yo-Nester cases.", "Text",
 CellChangeTimes->{{3.86096431193856*^9, 
  3.8609643257050734`*^9}},ExpressionUUID->"9fca2245-dec4-4074-b2c4-\
d4d77a0b72a0"],

Cell[BoxData[
 RowBox[{"(*", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"JobsBatch", "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\"\<spin_0p\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Alp1", "==", "0"}], ",", 
          RowBox[{"Alp2", "==", "0"}], ",", 
          RowBox[{"Alp3", "==", "0"}], ",", 
          RowBox[{"Alp4", "==", "0"}], ",", 
          RowBox[{"Alp5", "==", "0"}], ",", 
          RowBox[{
           RowBox[{
            RowBox[{"2", "Bet1"}], "+", "Bet2"}], "==", "0"}], ",", 
          RowBox[{
           RowBox[{"Bet1", "+", 
            RowBox[{"2", "Bet3"}]}], "==", "0"}], ",", 
          RowBox[{"cAlp1", "==", "0"}], ",", 
          RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"\"\<spin_0m\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Alp1", "==", "0"}], ",", 
          RowBox[{"Alp2", "==", "0"}], ",", 
          RowBox[{"Alp4", "==", "0"}], ",", 
          RowBox[{"Alp5", "==", "0"}], ",", 
          RowBox[{"Alp6", "==", "0"}], ",", 
          RowBox[{
           RowBox[{
            RowBox[{"2", "Bet1"}], "+", "Bet2"}], "==", "0"}], ",", 
          RowBox[{
           RowBox[{"Bet1", "+", 
            RowBox[{"2", "Bet3"}]}], "==", "0"}], ",", 
          RowBox[{"cAlp1", "==", "0"}], ",", 
          RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"\"\<simple_spin_1p\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Alp1", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
          RowBox[{"Bet1", "==", "0"}], ",", 
          RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp1", "==", "0"}], ",", 
          RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"\"\<simple_spin_1m\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Alp1", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
          RowBox[{"Bet1", "==", "0"}], ",", 
          RowBox[{"Bet3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp1", "==", "0"}], ",", 
          RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\"\<simple_spin_2m\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp5", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
          RowBox[{"Bet1", "==", "0"}], ",", 
          RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
          RowBox[{"Bet3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp1", "==", "0"}], ",", 
          RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\"\<simple_spin_0-2m_a\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Alp1", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp5", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
          RowBox[{"Bet1", "==", "0"}], ",", 
          RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
          RowBox[{"Bet3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp1", "==", "0"}], ",", 
          RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"\"\<simple_spin_0-2m_b\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp5", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
          RowBox[{"Bet1", "==", "0"}], ",", 
          RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
          RowBox[{"Bet3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp1", "==", "0"}], ",", 
          RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}]}], "}"}]}], 
   ";"}], "\[IndentingNewLine]", "*)"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.8561134940016623`*^9, 3.856113563953445*^9}, {
   3.856113864152964*^9, 3.856113866735909*^9}, {3.856114686146227*^9, 
   3.856114722201364*^9}, {3.8561147534818153`*^9, 3.856114786785334*^9}, {
   3.8561150379530697`*^9, 3.8561150382066393`*^9}, {3.856115132575592*^9, 
   3.856115149455024*^9}, 3.856165332453063*^9, {3.856165412953887*^9, 
   3.856165493969346*^9}, {3.856165882551499*^9, 3.8561659681267014`*^9}, {
   3.856252550417321*^9, 3.856252550671973*^9}, {3.856253384785056*^9, 
   3.856253537238635*^9}, {3.856253871827731*^9, 3.856253912977112*^9}, {
   3.856266135351696*^9, 3.85626613949498*^9}, {3.856330112716134*^9, 
   3.856330148356846*^9}, {3.856334077806583*^9, 3.85633412540273*^9}, {
   3.8563374211690483`*^9, 3.8563375146333323`*^9}, {3.8563713031675777`*^9, 
   3.8563713066542683`*^9}, {3.856407380427725*^9, 3.856407391803412*^9}, {
   3.856407608764429*^9, 3.8564076196720247`*^9}, 3.856407752091467*^9, 
   3.856407826016653*^9, 3.856407902750139*^9, {3.85640794860667*^9, 
   3.8564079506166077`*^9}, {3.8564080331074467`*^9, 3.856408035278562*^9}, 
   3.856408749583098*^9, {3.8564088204491577`*^9, 3.856408851243552*^9}, {
   3.856408937610999*^9, 3.856408958571206*^9}, 3.856426594371249*^9, {
   3.856546771934794*^9, 3.856546780533009*^9}, {3.856547050566327*^9, 
   3.856547182411356*^9}, {3.8565472180334578`*^9, 3.8565472297545033`*^9}, {
   3.856547261019403*^9, 3.856547319535778*^9}, {3.856547389634677*^9, 
   3.856547415811049*^9}, {3.856547502452675*^9, 3.8565475402935333`*^9}, {
   3.856547935615954*^9, 3.856547996958641*^9}, {3.8565480427427683`*^9, 
   3.856548130126974*^9}, {3.856548174271737*^9, 3.856548238495077*^9}, {
   3.856548994305244*^9, 3.856549018994968*^9}, {3.856549639533609*^9, 
   3.856549677197238*^9}, {3.856550750202263*^9, 3.856550754237236*^9}, {
   3.856584484801955*^9, 3.856584543913227*^9}, {3.856584817941999*^9, 
   3.856584869979888*^9}, 3.856585073668594*^9, {3.85658620385732*^9, 
   3.856586205956044*^9}, {3.856588920521818*^9, 3.856588921557769*^9}, 
   3.856588960476514*^9, {3.856590039993527*^9, 3.85659006622727*^9}, {
   3.856593936282866*^9, 3.856593948507574*^9}, {3.856593995428607*^9, 
   3.856593999792821*^9}, {3.856594042031241*^9, 3.856594053246723*^9}, {
   3.856594086439999*^9, 3.856594226084935*^9}, {3.85659432821384*^9, 
   3.856594469940914*^9}, {3.856594500282078*^9, 3.856594544618071*^9}, {
   3.856594612259853*^9, 3.856594769070032*^9}, {3.856594873905396*^9, 
   3.856594956693438*^9}, {3.8565950194680347`*^9, 3.856595074756282*^9}, {
   3.856595284751771*^9, 3.856595285285953*^9}, {3.856595383481922*^9, 
   3.856595426680007*^9}, {3.856596034870791*^9, 3.8565960350298347`*^9}, {
   3.85659607209651*^9, 3.85659608586699*^9}, {3.8565965701300287`*^9, 
   3.856596638947892*^9}, {3.856596725375925*^9, 3.856596731990838*^9}, {
   3.856598251369217*^9, 3.856598258241246*^9}, {3.856598434570442*^9, 
   3.856598462529182*^9}, {3.85659873835382*^9, 3.85659873866574*^9}, 
   3.856598824236*^9, {3.856598867332992*^9, 3.856598962700162*^9}, {
   3.8565990452773323`*^9, 3.856599068467422*^9}, {3.8565991551085377`*^9, 
   3.8565993229007053`*^9}, {3.8565993536914263`*^9, 3.856599417248561*^9}, {
   3.856600138751534*^9, 3.8566001661422253`*^9}, {3.8566002060986834`*^9, 
   3.856600225334676*^9}, {3.856600544026388*^9, 3.8566005441484118`*^9}, {
   3.856600586409638*^9, 3.856600612088909*^9}, {3.856600663947249*^9, 
   3.856600689908874*^9}, {3.856600745930562*^9, 3.8566008026048613`*^9}, {
   3.8566010638589973`*^9, 3.856601092986932*^9}, {3.85660138015917*^9, 
   3.8566013877983093`*^9}, {3.856606601818824*^9, 3.856606632750181*^9}, {
   3.856606721016076*^9, 3.856606801080719*^9}, {3.856607075354292*^9, 
   3.8566070793089247`*^9}, {3.856607285519328*^9, 3.856607350667756*^9}, {
   3.8566074599074087`*^9, 3.85660763592397*^9}, {3.856615179405943*^9, 
   3.856615210040922*^9}, {3.856630195894566*^9, 3.856630253259329*^9}, {
   3.8566931344471893`*^9, 3.856693204813795*^9}, 3.856693305118594*^9, {
   3.856693563355632*^9, 3.856693567190674*^9}, {3.856693669447075*^9, 
   3.8566936779144287`*^9}, {3.8566938062708397`*^9, 3.856693808318098*^9}, {
   3.8566939398466473`*^9, 3.856693942230942*^9}, {3.856697869620206*^9, 
   3.856697873773686*^9}, {3.856698360189404*^9, 3.856698371872653*^9}, {
   3.856700875553402*^9, 3.8567008771913757`*^9}, {3.8567019896412363`*^9, 
   3.856701992163899*^9}, {3.856702058555892*^9, 3.856702093022097*^9}, {
   3.8567024693164377`*^9, 3.856702472672245*^9}, {3.856702662221097*^9, 
   3.8567026639724407`*^9}, {3.856702933353261*^9, 3.856702937571703*^9}, {
   3.856702969037169*^9, 3.8567029816901608`*^9}, {3.856703272825307*^9, 
   3.8567032770478907`*^9}, {3.856703434516992*^9, 3.856703449079845*^9}, {
   3.856703764231814*^9, 3.856703769679906*^9}, {3.85670523350788*^9, 
   3.856705255121399*^9}, {3.856705300802005*^9, 3.8567053440432377`*^9}, 
   3.8567053835308113`*^9, {3.856716776053294*^9, 3.856716798617839*^9}, {
   3.856717296675805*^9, 3.856717300454899*^9}, {3.856717455235383*^9, 
   3.856717459302205*^9}, {3.856717558228253*^9, 3.856717577490952*^9}, {
   3.856766079861846*^9, 3.856766199328062*^9}, {3.856768657046853*^9, 
   3.856768658811205*^9}, {3.856768689609551*^9, 3.8567687186879473`*^9}, {
   3.856847824059163*^9, 3.856847829756719*^9}, {3.856885056781955*^9, 
   3.8568850572425117`*^9}, {3.8568875962693453`*^9, 3.856887605235325*^9}, {
   3.8569364364172297`*^9, 3.856936438322406*^9}, {3.8569572256026897`*^9, 
   3.856957232048373*^9}, {3.858142966335279*^9, 3.858142967860476*^9}, {
   3.85818112692178*^9, 3.8581811377360287`*^9}, {3.858181271677498*^9, 
   3.8581812782539043`*^9}, 3.858181354071229*^9, {3.858181524199338*^9, 
   3.858181555061183*^9}, 3.858181602086733*^9, {3.858181671389921*^9, 
   3.858181758453566*^9}, {3.858181924587641*^9, 3.858181926451866*^9}, {
   3.858181962742115*^9, 3.858181972902639*^9}, {3.8581820085582523`*^9, 
   3.8581820428319693`*^9}, {3.858182313602854*^9, 3.858182318711783*^9}, 
   3.858182736132729*^9, {3.858182873156907*^9, 3.858182876713172*^9}, {
   3.858184058358448*^9, 3.858184060194497*^9}, {3.858184214826552*^9, 
   3.8581842270070047`*^9}, {3.858184821147771*^9, 3.858184822231308*^9}, {
   3.8581858194322453`*^9, 3.85818582401626*^9}, {3.8582267013467503`*^9, 
   3.8582267055196037`*^9}, 3.858228301091529*^9, {3.858229492565473*^9, 
   3.8582295062000647`*^9}, {3.8582296669434967`*^9, 
   3.8582297318074417`*^9}, {3.8582301400178213`*^9, 
   3.8582301519399233`*^9}, {3.8582307301490602`*^9, 3.858230772531424*^9}, {
   3.858231510513658*^9, 3.858231523827112*^9}, {3.858233464638513*^9, 
   3.8582334712864532`*^9}, {3.8582336759973717`*^9, 
   3.8582336788817043`*^9}, {3.8582343296727877`*^9, 3.858234401096449*^9}, 
   3.858234510434876*^9, {3.858235066237193*^9, 3.8582350738367023`*^9}, {
   3.858235202140679*^9, 3.858235220235373*^9}, {3.858235454699109*^9, 
   3.858235467892789*^9}, {3.858236200960024*^9, 3.858236217929487*^9}, {
   3.858236832559057*^9, 3.858236840867004*^9}, {3.858238575009941*^9, 
   3.85823861822012*^9}, {3.8582393686107492`*^9, 3.8582393946451683`*^9}, {
   3.858239531711216*^9, 3.858239538715068*^9}, {3.8582397821401167`*^9, 
   3.858239782385582*^9}, {3.858243501836114*^9, 3.858243533066533*^9}, 
   3.8582517677726316`*^9, {3.858253141465578*^9, 3.858253156004655*^9}, {
   3.858262523424046*^9, 3.8582625263262672`*^9}, {3.8582626132154093`*^9, 
   3.858262651990097*^9}, {3.858263093996423*^9, 3.858263107363203*^9}, {
   3.858269621613408*^9, 3.8582696456949043`*^9}, {3.858270163378757*^9, 
   3.858270173825705*^9}, {3.8582710242137823`*^9, 3.858271030036566*^9}, {
   3.858339575377823*^9, 3.858339606721662*^9}, {3.858342927878196*^9, 
   3.8583429658853207`*^9}, {3.858433836920229*^9, 3.8584338580792007`*^9}, {
   3.858444408381193*^9, 3.858444410492931*^9}, {3.858488493308608*^9, 
   3.858488497917211*^9}, {3.858488536588729*^9, 3.8584885507801733`*^9}, {
   3.858489530903399*^9, 3.858489533822534*^9}, {3.858490078702059*^9, 
   3.858490081194944*^9}, {3.8584935454650297`*^9, 3.858493546463683*^9}, {
   3.858493586904846*^9, 3.8584936478150787`*^9}, {3.858495601141667*^9, 
   3.858495607299471*^9}, {3.858496325240609*^9, 3.858496327438936*^9}, 
   3.858497286249983*^9, {3.858498767025363*^9, 3.858498795672372*^9}, {
   3.858499119271175*^9, 3.8584991240866756`*^9}, {3.858510475487196*^9, 
   3.8585104773659773`*^9}, {3.858510663777513*^9, 3.8585107203833313`*^9}, {
   3.858510766166922*^9, 3.858510819231451*^9}, {3.858514732542738*^9, 
   3.858514775339328*^9}, {3.858524787588656*^9, 3.858524805815624*^9}, {
   3.858586280769*^9, 3.858586320902526*^9}, {3.8586041766885843`*^9, 
   3.858604235656663*^9}, {3.858604268024601*^9, 3.8586043932366858`*^9}, {
   3.8586756463415127`*^9, 3.858675658869523*^9}, {3.858693513549828*^9, 
   3.858693520322974*^9}, {3.858707622178495*^9, 3.858707625882985*^9}, {
   3.858713068848999*^9, 3.858713111616528*^9}, {3.8587397670160303`*^9, 
   3.85873976882025*^9}, {3.8587675401932583`*^9, 3.858767541652359*^9}, {
   3.858832057428598*^9, 3.858832057926138*^9}, {3.858837232206874*^9, 
   3.858837233645618*^9}, {3.858853493013969*^9, 3.8588535088894873`*^9}, 
   3.858855124164269*^9, {3.8588564226327744`*^9, 3.858856431548901*^9}, 
   3.8589136332742987`*^9, 3.858919114696944*^9, 3.85893209743512*^9, 
   3.858945274840987*^9, {3.858952938922271*^9, 3.858952942944778*^9}, 
   3.8589531113849363`*^9, 3.858995404369176*^9, {3.8589956417225437`*^9, 
   3.858995644914462*^9}, {3.8590358762954283`*^9, 3.85903590700359*^9}, {
   3.859091432673786*^9, 3.8590914661578608`*^9}, 3.859117315940999*^9, 
   3.859146655027433*^9, {3.859172984765532*^9, 3.859172993171767*^9}, {
   3.8592159531931887`*^9, 3.859215953986575*^9}, {3.859617013949033*^9, 
   3.8596170867982264`*^9}, {3.85970281276968*^9, 3.85970281470197*^9}, {
   3.859742016589634*^9, 3.8597420216673613`*^9}, {3.8597835044637527`*^9, 
   3.8597835199839087`*^9}, {3.8597911002859983`*^9, 3.859791100724319*^9}, 
   3.860040870391541*^9, {3.860944114979669*^9, 3.860944123421249*^9}, {
   3.860944933263855*^9, 3.860944945175519*^9}, {3.8609449945269814`*^9, 
   3.860945023815143*^9}, {3.860946122451845*^9, 3.8609461255880117`*^9}, {
   3.860951081407662*^9, 3.860951112256353*^9}, {3.860952240732245*^9, 
   3.860952251988269*^9}, {3.860952330947336*^9, 3.860952331211688*^9}, {
   3.860958306928714*^9, 3.860958310488752*^9}, {3.860964166172598*^9, 
   3.860964166892364*^9}, {3.860964225686287*^9, 3.860964226675321*^9}, {
   3.86096426922003*^9, 3.86096427095472*^9}, {3.8609643310254793`*^9, 
   3.860964338561879*^9}, {3.8610770491410637`*^9, 3.861077050706936*^9}, {
   3.861119277854916*^9, 3.861119281630355*^9}, {3.86117635148932*^9, 
   3.861176352138424*^9}, {3.86203876561231*^9, 
   3.8620387675792513`*^9}},ExpressionUUID->"3e400bc4-d94f-4570-9120-\
2160240f64a0"],

Cell["\<\
Careful analysis of \[OpenCurlyDoubleQuote]Commissioning Survey\
\[CloseCurlyDoubleQuote] indicated that there were two cases of interest when \
targeting strong coupling. For these cases, we would additionally like to \
know about the velocities: this is a tough calculation, poorly parallelized, \
so we can do this on appcg.\
\>", "Text",
 CellChangeTimes->{{3.862038664182266*^9, 
  3.8620386896683283`*^9}},ExpressionUUID->"78bc138e-f02b-49d3-b41d-\
8f6649a93e02"],

Cell[BoxData[
 RowBox[{"(*", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"JobsBatch", "=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\"\<simple_spin_1p_A3A4A6B1B3_velocities\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Alp1", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp4", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
          RowBox[{"Bet1", "==", "0"}], ",", 
          RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp1", "==", "0"}], ",", 
          RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\"\<simple_spin_1p_A3A6B1B2B3_velocities\>\"", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Alp1", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp2", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp3", "\[Equal]", "0"}], ",", 
          RowBox[{"Alp6", "\[Equal]", "0"}], ",", 
          RowBox[{"Bet1", "==", "0"}], ",", 
          RowBox[{"Bet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp1", "==", "0"}], ",", 
          RowBox[{"cAlp2", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp3", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp4", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp5", "\[Equal]", "0"}], ",", 
          RowBox[{"cAlp6", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet1", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet2", "\[Equal]", "0"}], ",", 
          RowBox[{"cBet3", "\[Equal]", "0"}]}], "}"}]}], "}"}]}], "}"}]}], 
   ";"}], "\[IndentingNewLine]", "*)"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.8561134940016623`*^9, 3.856113563953445*^9}, {
   3.856113864152964*^9, 3.856113866735909*^9}, {3.856114686146227*^9, 
   3.856114722201364*^9}, {3.8561147534818153`*^9, 3.856114786785334*^9}, {
   3.8561150379530697`*^9, 3.8561150382066393`*^9}, {3.856115132575592*^9, 
   3.856115149455024*^9}, 3.856165332453063*^9, {3.856165412953887*^9, 
   3.856165493969346*^9}, {3.856165882551499*^9, 3.8561659681267014`*^9}, {
   3.856252550417321*^9, 3.856252550671973*^9}, {3.856253384785056*^9, 
   3.856253537238635*^9}, {3.856253871827731*^9, 3.856253912977112*^9}, {
   3.856266135351696*^9, 3.85626613949498*^9}, {3.856330112716134*^9, 
   3.856330148356846*^9}, {3.856334077806583*^9, 3.85633412540273*^9}, {
   3.8563374211690483`*^9, 3.8563375146333323`*^9}, {3.8563713031675777`*^9, 
   3.8563713066542683`*^9}, {3.856407380427725*^9, 3.856407391803412*^9}, {
   3.856407608764429*^9, 3.8564076196720247`*^9}, 3.856407752091467*^9, 
   3.856407826016653*^9, 3.856407902750139*^9, {3.85640794860667*^9, 
   3.8564079506166077`*^9}, {3.8564080331074467`*^9, 3.856408035278562*^9}, 
   3.856408749583098*^9, {3.8564088204491577`*^9, 3.856408851243552*^9}, {
   3.856408937610999*^9, 3.856408958571206*^9}, 3.856426594371249*^9, {
   3.856546771934794*^9, 3.856546780533009*^9}, {3.856547050566327*^9, 
   3.856547182411356*^9}, {3.8565472180334578`*^9, 3.8565472297545033`*^9}, {
   3.856547261019403*^9, 3.856547319535778*^9}, {3.856547389634677*^9, 
   3.856547415811049*^9}, {3.856547502452675*^9, 3.8565475402935333`*^9}, {
   3.856547935615954*^9, 3.856547996958641*^9}, {3.8565480427427683`*^9, 
   3.856548130126974*^9}, {3.856548174271737*^9, 3.856548238495077*^9}, {
   3.856548994305244*^9, 3.856549018994968*^9}, {3.856549639533609*^9, 
   3.856549677197238*^9}, {3.856550750202263*^9, 3.856550754237236*^9}, {
   3.856584484801955*^9, 3.856584543913227*^9}, {3.856584817941999*^9, 
   3.856584869979888*^9}, 3.856585073668594*^9, {3.85658620385732*^9, 
   3.856586205956044*^9}, {3.856588920521818*^9, 3.856588921557769*^9}, 
   3.856588960476514*^9, {3.856590039993527*^9, 3.85659006622727*^9}, {
   3.856593936282866*^9, 3.856593948507574*^9}, {3.856593995428607*^9, 
   3.856593999792821*^9}, {3.856594042031241*^9, 3.856594053246723*^9}, {
   3.856594086439999*^9, 3.856594226084935*^9}, {3.85659432821384*^9, 
   3.856594469940914*^9}, {3.856594500282078*^9, 3.856594544618071*^9}, {
   3.856594612259853*^9, 3.856594769070032*^9}, {3.856594873905396*^9, 
   3.856594956693438*^9}, {3.8565950194680347`*^9, 3.856595074756282*^9}, {
   3.856595284751771*^9, 3.856595285285953*^9}, {3.856595383481922*^9, 
   3.856595426680007*^9}, {3.856596034870791*^9, 3.8565960350298347`*^9}, {
   3.85659607209651*^9, 3.85659608586699*^9}, {3.8565965701300287`*^9, 
   3.856596638947892*^9}, {3.856596725375925*^9, 3.856596731990838*^9}, {
   3.856598251369217*^9, 3.856598258241246*^9}, {3.856598434570442*^9, 
   3.856598462529182*^9}, {3.85659873835382*^9, 3.85659873866574*^9}, 
   3.856598824236*^9, {3.856598867332992*^9, 3.856598962700162*^9}, {
   3.8565990452773323`*^9, 3.856599068467422*^9}, {3.8565991551085377`*^9, 
   3.8565993229007053`*^9}, {3.8565993536914263`*^9, 3.856599417248561*^9}, {
   3.856600138751534*^9, 3.8566001661422253`*^9}, {3.8566002060986834`*^9, 
   3.856600225334676*^9}, {3.856600544026388*^9, 3.8566005441484118`*^9}, {
   3.856600586409638*^9, 3.856600612088909*^9}, {3.856600663947249*^9, 
   3.856600689908874*^9}, {3.856600745930562*^9, 3.8566008026048613`*^9}, {
   3.8566010638589973`*^9, 3.856601092986932*^9}, {3.85660138015917*^9, 
   3.8566013877983093`*^9}, {3.856606601818824*^9, 3.856606632750181*^9}, {
   3.856606721016076*^9, 3.856606801080719*^9}, {3.856607075354292*^9, 
   3.8566070793089247`*^9}, {3.856607285519328*^9, 3.856607350667756*^9}, {
   3.8566074599074087`*^9, 3.85660763592397*^9}, {3.856615179405943*^9, 
   3.856615210040922*^9}, {3.856630195894566*^9, 3.856630253259329*^9}, {
   3.8566931344471893`*^9, 3.856693204813795*^9}, 3.856693305118594*^9, {
   3.856693563355632*^9, 3.856693567190674*^9}, {3.856693669447075*^9, 
   3.8566936779144287`*^9}, {3.8566938062708397`*^9, 3.856693808318098*^9}, {
   3.8566939398466473`*^9, 3.856693942230942*^9}, {3.856697869620206*^9, 
   3.856697873773686*^9}, {3.856698360189404*^9, 3.856698371872653*^9}, {
   3.856700875553402*^9, 3.8567008771913757`*^9}, {3.8567019896412363`*^9, 
   3.856701992163899*^9}, {3.856702058555892*^9, 3.856702093022097*^9}, {
   3.8567024693164377`*^9, 3.856702472672245*^9}, {3.856702662221097*^9, 
   3.8567026639724407`*^9}, {3.856702933353261*^9, 3.856702937571703*^9}, {
   3.856702969037169*^9, 3.8567029816901608`*^9}, {3.856703272825307*^9, 
   3.8567032770478907`*^9}, {3.856703434516992*^9, 3.856703449079845*^9}, {
   3.856703764231814*^9, 3.856703769679906*^9}, {3.85670523350788*^9, 
   3.856705255121399*^9}, {3.856705300802005*^9, 3.8567053440432377`*^9}, 
   3.8567053835308113`*^9, {3.856716776053294*^9, 3.856716798617839*^9}, {
   3.856717296675805*^9, 3.856717300454899*^9}, {3.856717455235383*^9, 
   3.856717459302205*^9}, {3.856717558228253*^9, 3.856717577490952*^9}, {
   3.856766079861846*^9, 3.856766199328062*^9}, {3.856768657046853*^9, 
   3.856768658811205*^9}, {3.856768689609551*^9, 3.8567687186879473`*^9}, {
   3.856847824059163*^9, 3.856847829756719*^9}, {3.856885056781955*^9, 
   3.8568850572425117`*^9}, {3.8568875962693453`*^9, 3.856887605235325*^9}, {
   3.8569364364172297`*^9, 3.856936438322406*^9}, {3.8569572256026897`*^9, 
   3.856957232048373*^9}, {3.858142966335279*^9, 3.858142967860476*^9}, {
   3.85818112692178*^9, 3.8581811377360287`*^9}, {3.858181271677498*^9, 
   3.8581812782539043`*^9}, 3.858181354071229*^9, {3.858181524199338*^9, 
   3.858181555061183*^9}, 3.858181602086733*^9, {3.858181671389921*^9, 
   3.858181758453566*^9}, {3.858181924587641*^9, 3.858181926451866*^9}, {
   3.858181962742115*^9, 3.858181972902639*^9}, {3.8581820085582523`*^9, 
   3.8581820428319693`*^9}, {3.858182313602854*^9, 3.858182318711783*^9}, 
   3.858182736132729*^9, {3.858182873156907*^9, 3.858182876713172*^9}, {
   3.858184058358448*^9, 3.858184060194497*^9}, {3.858184214826552*^9, 
   3.8581842270070047`*^9}, {3.858184821147771*^9, 3.858184822231308*^9}, {
   3.8581858194322453`*^9, 3.85818582401626*^9}, {3.8582267013467503`*^9, 
   3.8582267055196037`*^9}, 3.858228301091529*^9, {3.858229492565473*^9, 
   3.8582295062000647`*^9}, {3.8582296669434967`*^9, 
   3.8582297318074417`*^9}, {3.8582301400178213`*^9, 
   3.8582301519399233`*^9}, {3.8582307301490602`*^9, 3.858230772531424*^9}, {
   3.858231510513658*^9, 3.858231523827112*^9}, {3.858233464638513*^9, 
   3.8582334712864532`*^9}, {3.8582336759973717`*^9, 
   3.8582336788817043`*^9}, {3.8582343296727877`*^9, 3.858234401096449*^9}, 
   3.858234510434876*^9, {3.858235066237193*^9, 3.8582350738367023`*^9}, {
   3.858235202140679*^9, 3.858235220235373*^9}, {3.858235454699109*^9, 
   3.858235467892789*^9}, {3.858236200960024*^9, 3.858236217929487*^9}, {
   3.858236832559057*^9, 3.858236840867004*^9}, {3.858238575009941*^9, 
   3.85823861822012*^9}, {3.8582393686107492`*^9, 3.8582393946451683`*^9}, {
   3.858239531711216*^9, 3.858239538715068*^9}, {3.8582397821401167`*^9, 
   3.858239782385582*^9}, {3.858243501836114*^9, 3.858243533066533*^9}, 
   3.8582517677726316`*^9, {3.858253141465578*^9, 3.858253156004655*^9}, {
   3.858262523424046*^9, 3.8582625263262672`*^9}, {3.8582626132154093`*^9, 
   3.858262651990097*^9}, {3.858263093996423*^9, 3.858263107363203*^9}, {
   3.858269621613408*^9, 3.8582696456949043`*^9}, {3.858270163378757*^9, 
   3.858270173825705*^9}, {3.8582710242137823`*^9, 3.858271030036566*^9}, {
   3.858339575377823*^9, 3.858339606721662*^9}, {3.858342927878196*^9, 
   3.8583429658853207`*^9}, {3.858433836920229*^9, 3.8584338580792007`*^9}, {
   3.858444408381193*^9, 3.858444410492931*^9}, {3.858488493308608*^9, 
   3.858488497917211*^9}, {3.858488536588729*^9, 3.8584885507801733`*^9}, {
   3.858489530903399*^9, 3.858489533822534*^9}, {3.858490078702059*^9, 
   3.858490081194944*^9}, {3.8584935454650297`*^9, 3.858493546463683*^9}, {
   3.858493586904846*^9, 3.8584936478150787`*^9}, {3.858495601141667*^9, 
   3.858495607299471*^9}, {3.858496325240609*^9, 3.858496327438936*^9}, 
   3.858497286249983*^9, {3.858498767025363*^9, 3.858498795672372*^9}, {
   3.858499119271175*^9, 3.8584991240866756`*^9}, {3.858510475487196*^9, 
   3.8585104773659773`*^9}, {3.858510663777513*^9, 3.8585107203833313`*^9}, {
   3.858510766166922*^9, 3.858510819231451*^9}, {3.858514732542738*^9, 
   3.858514775339328*^9}, {3.858524787588656*^9, 3.858524805815624*^9}, {
   3.858586280769*^9, 3.858586320902526*^9}, {3.8586041766885843`*^9, 
   3.858604235656663*^9}, {3.858604268024601*^9, 3.8586043932366858`*^9}, {
   3.8586756463415127`*^9, 3.858675658869523*^9}, {3.858693513549828*^9, 
   3.858693520322974*^9}, {3.858707622178495*^9, 3.858707625882985*^9}, {
   3.858713068848999*^9, 3.858713111616528*^9}, {3.8587397670160303`*^9, 
   3.85873976882025*^9}, {3.8587675401932583`*^9, 3.858767541652359*^9}, {
   3.858832057428598*^9, 3.858832057926138*^9}, {3.858837232206874*^9, 
   3.858837233645618*^9}, {3.858853493013969*^9, 3.8588535088894873`*^9}, 
   3.858855124164269*^9, {3.8588564226327744`*^9, 3.858856431548901*^9}, 
   3.8589136332742987`*^9, 3.858919114696944*^9, 3.85893209743512*^9, 
   3.858945274840987*^9, {3.858952938922271*^9, 3.858952942944778*^9}, 
   3.8589531113849363`*^9, 3.858995404369176*^9, {3.8589956417225437`*^9, 
   3.858995644914462*^9}, {3.8590358762954283`*^9, 3.85903590700359*^9}, {
   3.859091432673786*^9, 3.8590914661578608`*^9}, 3.859117315940999*^9, 
   3.859146655027433*^9, {3.859172984765532*^9, 3.859172993171767*^9}, {
   3.8592159531931887`*^9, 3.859215953986575*^9}, {3.859617013949033*^9, 
   3.8596170867982264`*^9}, {3.85970281276968*^9, 3.85970281470197*^9}, {
   3.859742016589634*^9, 3.8597420216673613`*^9}, {3.8597835044637527`*^9, 
   3.8597835199839087`*^9}, {3.8597911002859983`*^9, 3.859791100724319*^9}, 
   3.860040870391541*^9, {3.860944114979669*^9, 3.860944123421249*^9}, {
   3.860944933263855*^9, 3.860944945175519*^9}, {3.8609449945269814`*^9, 
   3.860945023815143*^9}, {3.860946122451845*^9, 3.8609461255880117`*^9}, {
   3.860951081407662*^9, 3.860951112256353*^9}, {3.860952240732245*^9, 
   3.860952251988269*^9}, {3.860952330947336*^9, 3.860952331211688*^9}, {
   3.860958306928714*^9, 3.860958310488752*^9}, {3.860964166172598*^9, 
   3.860964166892364*^9}, {3.860964225686287*^9, 3.860964226675321*^9}, {
   3.86096426922003*^9, 3.86096427095472*^9}, {3.861077044060281*^9, 
   3.8610770548677607`*^9}, {3.861119273806775*^9, 3.861119275286681*^9}, {
   3.862038588693138*^9, 3.8620385888264523`*^9}, {3.862038742226532*^9, 
   3.862038790762507*^9}, {3.86203937529296*^9, 3.862039376420928*^9}, {
   3.8623062616489973`*^9, 
   3.862306265521446*^9}},ExpressionUUID->"e63ef68a-f17d-4c30-9333-\
5719bba7407b"],

Cell["This is where we do the job. ", "Text",
 CellChangeTimes->{{3.862038820758507*^9, 
  3.8620388235886307`*^9}},ExpressionUUID->"b36e3c49-01c9-429c-bc1a-\
47bee07aef49"],

Cell[BoxData[
 RowBox[{
  RowBox[{"StudyTheory", "[", 
   RowBox[{"JobsBatch", ",", 
    RowBox[{"\"\<Import\>\"", "\[Rule]", "False"}], ",", 
    RowBox[{"\"\<Brackets\>\"", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",\

 InitializationCell->True,
 CellChangeTimes->{
  3.861077037956285*^9, {3.8615315216941023`*^9, 3.861531530493895*^9}, {
   3.86230804896756*^9, 
   3.862308049998921*^9}},ExpressionUUID->"69ca6540-929e-4ea1-8892-\
d560ff402d8c"],

Cell["Kill the master kernel.", "Text",
 CellChangeTimes->{{3.862038832137828*^9, 
  3.8620388352258377`*^9}},ExpressionUUID->"6ef3c73b-f594-4af2-a18b-\
aada82fb5018"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Quit", "[", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.8561134940016623`*^9, 3.856113563953445*^9}, {
   3.856113864152964*^9, 3.856113866735909*^9}, {3.856114686146227*^9, 
   3.856114722201364*^9}, {3.8561147534818153`*^9, 3.856114786785334*^9}, {
   3.8561150379530697`*^9, 3.8561150382066393`*^9}, {3.856115132575592*^9, 
   3.856115149455024*^9}, 3.856165332453063*^9, {3.856165412953887*^9, 
   3.856165493969346*^9}, {3.856165882551499*^9, 3.8561659681267014`*^9}, {
   3.856252550417321*^9, 3.856252550671973*^9}, {3.856253384785056*^9, 
   3.856253537238635*^9}, {3.856253871827731*^9, 3.856253912977112*^9}, {
   3.856266135351696*^9, 3.85626613949498*^9}, {3.856330112716134*^9, 
   3.856330148356846*^9}, {3.856334077806583*^9, 3.85633412540273*^9}, {
   3.8563374211690483`*^9, 3.8563375146333323`*^9}, {3.8563713031675777`*^9, 
   3.8563713066542683`*^9}, {3.856407380427725*^9, 3.856407391803412*^9}, {
   3.856407608764429*^9, 3.8564076196720247`*^9}, 3.856407752091467*^9, 
   3.856407826016653*^9, 3.856407902750139*^9, {3.85640794860667*^9, 
   3.8564079506166077`*^9}, {3.8564080331074467`*^9, 3.856408035278562*^9}, 
   3.856408749583098*^9, {3.8564088204491577`*^9, 3.856408851243552*^9}, {
   3.856408937610999*^9, 3.856408958571206*^9}, 3.856426594371249*^9, {
   3.856546771934794*^9, 3.856546780533009*^9}, {3.856547050566327*^9, 
   3.856547182411356*^9}, {3.8565472180334578`*^9, 3.8565472297545033`*^9}, {
   3.856547261019403*^9, 3.856547319535778*^9}, {3.856547389634677*^9, 
   3.856547415811049*^9}, {3.856547502452675*^9, 3.8565475402935333`*^9}, {
   3.856547935615954*^9, 3.856547996958641*^9}, {3.8565480427427683`*^9, 
   3.856548130126974*^9}, {3.856548174271737*^9, 3.856548238495077*^9}, {
   3.856548994305244*^9, 3.856549018994968*^9}, {3.856549639533609*^9, 
   3.856549677197238*^9}, {3.856550750202263*^9, 3.856550754237236*^9}, {
   3.856584484801955*^9, 3.856584543913227*^9}, {3.856584817941999*^9, 
   3.856584869979888*^9}, 3.856585073668594*^9, {3.85658620385732*^9, 
   3.856586205956044*^9}, {3.856588920521818*^9, 3.856588921557769*^9}, 
   3.856588960476514*^9, {3.856590039993527*^9, 3.85659006622727*^9}, {
   3.856593936282866*^9, 3.856593948507574*^9}, {3.856593995428607*^9, 
   3.856593999792821*^9}, {3.856594042031241*^9, 3.856594053246723*^9}, {
   3.856594086439999*^9, 3.856594226084935*^9}, {3.85659432821384*^9, 
   3.856594469940914*^9}, {3.856594500282078*^9, 3.856594544618071*^9}, {
   3.856594612259853*^9, 3.856594769070032*^9}, {3.856594873905396*^9, 
   3.856594956693438*^9}, {3.8565950194680347`*^9, 3.856595074756282*^9}, {
   3.856595284751771*^9, 3.856595285285953*^9}, {3.856595383481922*^9, 
   3.856595426680007*^9}, {3.856596034870791*^9, 3.8565960350298347`*^9}, {
   3.85659607209651*^9, 3.85659608586699*^9}, {3.8565965701300287`*^9, 
   3.856596638947892*^9}, {3.856596725375925*^9, 3.856596731990838*^9}, {
   3.856598251369217*^9, 3.856598258241246*^9}, {3.856598434570442*^9, 
   3.856598462529182*^9}, {3.85659873835382*^9, 3.85659873866574*^9}, 
   3.856598824236*^9, {3.856598867332992*^9, 3.856598962700162*^9}, {
   3.8565990452773323`*^9, 3.856599068467422*^9}, {3.8565991551085377`*^9, 
   3.8565993229007053`*^9}, {3.8565993536914263`*^9, 3.856599417248561*^9}, {
   3.856600138751534*^9, 3.8566001661422253`*^9}, {3.8566002060986834`*^9, 
   3.856600225334676*^9}, {3.856600544026388*^9, 3.8566005441484118`*^9}, {
   3.856600586409638*^9, 3.856600612088909*^9}, {3.856600663947249*^9, 
   3.856600689908874*^9}, {3.856600745930562*^9, 3.8566008026048613`*^9}, {
   3.8566010638589973`*^9, 3.856601092986932*^9}, {3.85660138015917*^9, 
   3.8566013877983093`*^9}, {3.856606601818824*^9, 3.856606632750181*^9}, {
   3.856606721016076*^9, 3.856606801080719*^9}, {3.856607075354292*^9, 
   3.8566070793089247`*^9}, {3.856607285519328*^9, 3.856607350667756*^9}, {
   3.8566074599074087`*^9, 3.85660763592397*^9}, {3.856615179405943*^9, 
   3.856615210040922*^9}, {3.856630195894566*^9, 3.856630253259329*^9}, {
   3.8566931344471893`*^9, 3.856693204813795*^9}, 3.856693305118594*^9, {
   3.856693563355632*^9, 3.856693567190674*^9}, {3.856693669447075*^9, 
   3.8566936779144287`*^9}, {3.8566938062708397`*^9, 3.856693808318098*^9}, {
   3.8566939398466473`*^9, 3.856693942230942*^9}, {3.856697869620206*^9, 
   3.856697873773686*^9}, {3.856698360189404*^9, 3.856698371872653*^9}, {
   3.856700875553402*^9, 3.8567008771913757`*^9}, {3.8567019896412363`*^9, 
   3.856701992163899*^9}, {3.856702058555892*^9, 3.856702093022097*^9}, {
   3.8567024693164377`*^9, 3.856702472672245*^9}, {3.856702662221097*^9, 
   3.8567026639724407`*^9}, {3.856702933353261*^9, 3.856702937571703*^9}, {
   3.856702969037169*^9, 3.8567029816901608`*^9}, {3.856703272825307*^9, 
   3.8567032770478907`*^9}, {3.856703434516992*^9, 3.856703449079845*^9}, {
   3.856703764231814*^9, 3.856703769679906*^9}, {3.85670523350788*^9, 
   3.856705255121399*^9}, {3.856705300802005*^9, 3.8567053440432377`*^9}, 
   3.8567053835308113`*^9, {3.856716776053294*^9, 3.856716798617839*^9}, {
   3.856717296675805*^9, 3.856717300454899*^9}, {3.856717455235383*^9, 
   3.856717459302205*^9}, {3.856717558228253*^9, 3.856717577490952*^9}, {
   3.856766079861846*^9, 3.856766199328062*^9}, {3.856768657046853*^9, 
   3.856768658811205*^9}, {3.856768689609551*^9, 3.8567687186879473`*^9}, {
   3.856847824059163*^9, 3.856847829756719*^9}, {3.856885056781955*^9, 
   3.8568850572425117`*^9}, {3.8568875962693453`*^9, 3.856887605235325*^9}, {
   3.8569364364172297`*^9, 3.856936438322406*^9}, {3.8569572256026897`*^9, 
   3.856957232048373*^9}, {3.858142966335279*^9, 3.858142967860476*^9}, {
   3.85818112692178*^9, 3.8581811377360287`*^9}, {3.858181271677498*^9, 
   3.8581812782539043`*^9}, 3.858181354071229*^9, {3.858181524199338*^9, 
   3.858181555061183*^9}, 3.858181602086733*^9, {3.858181671389921*^9, 
   3.858181758453566*^9}, {3.858181924587641*^9, 3.858181926451866*^9}, {
   3.858181962742115*^9, 3.858181972902639*^9}, {3.8581820085582523`*^9, 
   3.8581820428319693`*^9}, {3.858182313602854*^9, 3.858182318711783*^9}, 
   3.858182736132729*^9, {3.858182873156907*^9, 3.858182876713172*^9}, {
   3.858184058358448*^9, 3.858184060194497*^9}, {3.858184214826552*^9, 
   3.8581842270070047`*^9}, {3.858184821147771*^9, 3.858184822231308*^9}, {
   3.8581858194322453`*^9, 3.85818582401626*^9}, {3.8582267013467503`*^9, 
   3.8582267055196037`*^9}, 3.858228301091529*^9, {3.858229492565473*^9, 
   3.8582295062000647`*^9}, {3.8582296669434967`*^9, 
   3.8582297318074417`*^9}, {3.8582301400178213`*^9, 
   3.8582301519399233`*^9}, {3.8582307301490602`*^9, 3.858230772531424*^9}, {
   3.858231510513658*^9, 3.858231523827112*^9}, {3.858233464638513*^9, 
   3.8582334712864532`*^9}, {3.8582336759973717`*^9, 
   3.8582336788817043`*^9}, {3.8582343296727877`*^9, 3.858234401096449*^9}, 
   3.858234510434876*^9, {3.858235066237193*^9, 3.8582350738367023`*^9}, {
   3.858235202140679*^9, 3.858235220235373*^9}, {3.858235454699109*^9, 
   3.858235467892789*^9}, {3.858236200960024*^9, 3.858236217929487*^9}, {
   3.858236832559057*^9, 3.858236840867004*^9}, {3.858238575009941*^9, 
   3.85823861822012*^9}, {3.8582393686107492`*^9, 3.8582393946451683`*^9}, {
   3.858239531711216*^9, 3.858239538715068*^9}, {3.8582397821401167`*^9, 
   3.858239782385582*^9}, {3.858243501836114*^9, 3.858243533066533*^9}, 
   3.8582517677726316`*^9, {3.858253141465578*^9, 3.858253156004655*^9}, {
   3.858262523424046*^9, 3.8582625263262672`*^9}, {3.8582626132154093`*^9, 
   3.858262651990097*^9}, {3.858263093996423*^9, 3.858263107363203*^9}, {
   3.858269621613408*^9, 3.8582696456949043`*^9}, {3.858270163378757*^9, 
   3.858270173825705*^9}, {3.8582710242137823`*^9, 3.858271030036566*^9}, {
   3.858339575377823*^9, 3.858339606721662*^9}, {3.858342927878196*^9, 
   3.8583429658853207`*^9}, {3.858433836920229*^9, 3.8584338580792007`*^9}, {
   3.858444408381193*^9, 3.858444410492931*^9}, {3.858488493308608*^9, 
   3.858488497917211*^9}, {3.858488536588729*^9, 3.8584885507801733`*^9}, {
   3.858489530903399*^9, 3.858489533822534*^9}, {3.858490078702059*^9, 
   3.858490081194944*^9}, {3.8584935454650297`*^9, 3.858493546463683*^9}, {
   3.858493586904846*^9, 3.8584936478150787`*^9}, {3.858495601141667*^9, 
   3.858495607299471*^9}, {3.858496325240609*^9, 3.858496327438936*^9}, 
   3.858497286249983*^9, {3.858498767025363*^9, 3.858498795672372*^9}, {
   3.858499119271175*^9, 3.8584991240866756`*^9}, {3.858510475487196*^9, 
   3.8585104773659773`*^9}, {3.858510663777513*^9, 3.8585107203833313`*^9}, {
   3.858510766166922*^9, 3.858510819231451*^9}, {3.858514732542738*^9, 
   3.858514775339328*^9}, {3.858524787588656*^9, 3.858524805815624*^9}, {
   3.858586280769*^9, 3.858586320902526*^9}, {3.8586041766885843`*^9, 
   3.858604235656663*^9}, {3.858604268024601*^9, 3.8586043932366858`*^9}, {
   3.8586756463415127`*^9, 3.858675658869523*^9}, {3.858693513549828*^9, 
   3.858693520322974*^9}, {3.858707622178495*^9, 3.858707625882985*^9}, {
   3.858713068848999*^9, 3.858713111616528*^9}, {3.8587397670160303`*^9, 
   3.85873976882025*^9}, {3.8587675401932583`*^9, 3.858767541652359*^9}, {
   3.858832057428598*^9, 3.858832057926138*^9}, {3.858837232206874*^9, 
   3.858837233645618*^9}, {3.858853493013969*^9, 3.8588535088894873`*^9}, 
   3.858855124164269*^9, {3.8588564226327744`*^9, 3.858856431548901*^9}, 
   3.8589136332742987`*^9, 3.858919114696944*^9, 3.85893209743512*^9, 
   3.858945274840987*^9, {3.858952938922271*^9, 3.858952942944778*^9}, 
   3.8589531113849363`*^9, 3.858995404369176*^9, {3.8589956417225437`*^9, 
   3.858995644914462*^9}, {3.8590358762954283`*^9, 3.85903590700359*^9}, {
   3.859091432673786*^9, 3.8590914661578608`*^9}, 3.859117315940999*^9, 
   3.859146655027433*^9, {3.859172984765532*^9, 3.859172993171767*^9}, {
   3.8592159531931887`*^9, 3.859215953986575*^9}, {3.859617013949033*^9, 
   3.8596170867982264`*^9}, {3.85970281276968*^9, 3.85970281470197*^9}, {
   3.859742016589634*^9, 3.8597420216673613`*^9}, {3.8597835044637527`*^9, 
   3.8597835199839087`*^9}, {3.8597911002859983`*^9, 3.859791100724319*^9}, 
   3.860040870391541*^9, {3.860944114979669*^9, 3.860944123421249*^9}, {
   3.860944933263855*^9, 3.860944945175519*^9}, {3.8609449945269814`*^9, 
   3.860945023815143*^9}, {3.860946122451845*^9, 3.8609461255880117`*^9}, {
   3.860951081407662*^9, 3.860951112256353*^9}, {3.860952240732245*^9, 
   3.860952251988269*^9}, {3.860952330947336*^9, 3.860952331211688*^9}, {
   3.860958306928714*^9, 3.860958310488752*^9}, {3.860964166172598*^9, 
   3.860964166892364*^9}, {3.860964225686287*^9, 3.860964226675321*^9}, {
   3.86096426922003*^9, 3.86096427095472*^9}, {3.8609643310254793`*^9, 
   3.860964338561879*^9}},ExpressionUUID->"c807b70a-05e7-4ce9-b686-\
0a1ee91bb893"]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{1920, 1147},
WindowMargins->{{Automatic, 1080}, {Automatic, 53}},
TaggingRules->Association["TryRealOnly" -> False],
FrontEndVersion->"11.3 for Linux x86 (64-bit) (March 6, 2018)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 352, 9, 99, "Title",ExpressionUUID->"77b3805d-060b-4070-8508-41576cdf0004"],
Cell[935, 33, 370, 8, 36, "Text",ExpressionUUID->"ac3b952a-323b-405e-9ac1-f072cdc53b32"],
Cell[1308, 43, 394, 8, 36, "Text",ExpressionUUID->"c9fedf3e-88c2-4938-860c-5810784e7e18"],
Cell[1705, 53, 280, 7, 47, "Input",ExpressionUUID->"9966b0f6-fed7-4946-93a6-692cd3f2588c",
 InitializationCell->True],
Cell[1988, 62, 285, 6, 36, "Text",ExpressionUUID->"d26fa4bd-c3f3-4a56-9cd8-37b9ae8b758b"],
Cell[2276, 70, 265, 6, 47, "Input",ExpressionUUID->"fd726b7b-eb57-4386-8546-8b40abb5b147",
 InitializationCell->True],
Cell[2544, 78, 169, 3, 36, "Text",ExpressionUUID->"28427427-46cf-4df7-8bfc-e4b50069fca9"],
Cell[2716, 83, 875, 14, 140, "Input",ExpressionUUID->"ee8cde45-7297-47ec-8e7b-ec674b3b00cf",
 InitializationCell->True],
Cell[3594, 99, 163, 3, 36, "Text",ExpressionUUID->"38aec83e-3336-41f1-ac4d-fd2593bf2789"],
Cell[3760, 104, 11092, 159, 119, "Input",ExpressionUUID->"ecfae8f8-9928-4519-ae15-3f06f02b8c48",
 InitializationCell->True],
Cell[14855, 265, 167, 3, 36, "Text",ExpressionUUID->"a8923ab6-e09d-46fe-bae0-5887d4929536"],
Cell[15025, 270, 12234, 182, 94, "Input",ExpressionUUID->"ffea50e1-9566-4d73-8e16-9677a199ce2a",
 InitializationCell->True],
Cell[27262, 454, 159, 3, 36, "Text",ExpressionUUID->"9fca2245-dec4-4074-b2c4-d4d77a0b72a0"],
Cell[27424, 459, 18115, 311, 232, "Input",ExpressionUUID->"3e400bc4-d94f-4570-9120-2160240f64a0",
 InitializationCell->True],
Cell[45542, 772, 479, 9, 36, "Text",ExpressionUUID->"78bc138e-f02b-49d3-b41d-8f6649a93e02"],
Cell[46024, 783, 13021, 196, 117, "Input",ExpressionUUID->"e63ef68a-f17d-4c30-9333-5719bba7407b",
 InitializationCell->True],
Cell[59048, 981, 173, 3, 36, "Text",ExpressionUUID->"b36e3c49-01c9-429c-bc1a-47bee07aef49"],
Cell[59224, 986, 453, 12, 47, "Input",ExpressionUUID->"69ca6540-929e-4ea1-8892-d560ff402d8c",
 InitializationCell->True],
Cell[59680, 1000, 167, 3, 36, "Text",ExpressionUUID->"6ef3c73b-f594-4af2-a18b-aada82fb5018"],
Cell[59850, 1005, 10843, 150, 47, "Input",ExpressionUUID->"c807b70a-05e7-4ce9-b686-0a1ee91bb893",
 InitializationCell->True]
}, Open  ]]
}
]
*)

